/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.jca;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.mail.Session;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

import org.apache.log4j.Category;

/**
 *
 * @author  Scott.Stark@jboss.org
 * @version $Revision: 1.1.1.1 $
 */
public class ManagedWrappedSession implements ManagedConnection {
	static Category log = Category.getInstance(ManagedWrappedSession.class);
	ArrayList listeners = new ArrayList();
	Session session = null;
	private Set handles = new HashSet();
	
	private Properties props = null;

	protected boolean destroyed = false;

	private WrappedSessionManagedSessionFactory mcf;

	/** Creates new FSManagedConnection */
	public ManagedWrappedSession(
		WrappedSessionManagedSessionFactory mcf,
		Session session,
		Properties props) {
			
		this.mcf = mcf;
		this.session = session;
		this.props = props;
		
		log.debug("ctor, session=" + session + ", props=" + props);
	}
	
	public void addConnectionEventListener(ConnectionEventListener connectionEventListener) {

		synchronized (listeners) {
			listeners.add(connectionEventListener);
		}

	}
	public void removeConnectionEventListener(ConnectionEventListener connectionEventListener) {
		synchronized (listeners) {
			listeners.remove(connectionEventListener);
		}
	}
	public void associateConnection(Object handle) throws ResourceException {
		if (!(handle instanceof WrappedSession)) {
			throw new ResourceException("Wrong type on associateConnection for MailSession JCA");
		}
		WrappedSession session = (WrappedSession) handle;
		session.setManagedConnection(this);

	}
	public void cleanup() throws ResourceException {
		synchronized (handles) {
			for (Iterator iter = handles.iterator(); iter.hasNext();) {
				WrappedSession element = (WrappedSession) iter.next();
				element.setManagedConnection(null);
			}
			handles.clear();
		}
	}
	public void destroy() throws ResourceException {
		destroyed = true;

		cleanup();
	}

	public Object getConnection(Subject subject, ConnectionRequestInfo info)
		throws ResourceException {

		checkIdentity(subject, info);
		WrappedSession lc = new WrappedSession(this);
		synchronized (handles) {
			handles.add(lc);
		}
		return lc;
	}
	public LocalTransaction getLocalTransaction() throws ResourceException {
		log.debug("getLocalTransaction");
		return null;
	}
	public ManagedConnectionMetaData getMetaData() throws ResourceException {
		log.debug("getMetaData");
		return new ManagedWrappedSessionMetaData();
	}
	public XAResource getXAResource() throws ResourceException {
		log.debug("getXAResource");
		return null;
	}
	public PrintWriter getLogWriter() throws ResourceException {
		return null;
	}
	public void setLogWriter(PrintWriter out) throws ResourceException {
	}

	void closeHandle(WrappedSession handle) {
		if (destroyed)
			return;

		synchronized (handles) {
			handles.remove(handle);
		}
		ConnectionEvent ce =
			new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
		ce.setConnectionHandle(handle);
		ArrayList copy = null;
		synchronized (listeners) {
			copy = new ArrayList(listeners);
		}
		for (Iterator i = copy.iterator(); i.hasNext();) {
			ConnectionEventListener cel = (ConnectionEventListener) i.next();
			cel.connectionClosed(ce);
		}
	}

	/**
	 * Describe <code>checkIdentity</code> method here.
	 *
	 * @exception ResourceException if an error occurs
	 * @todo check if subject or cri should have higher priority.
	 */
	private void checkIdentity(Subject subject, ConnectionRequestInfo cri)
	   throws ResourceException
	{
	   Properties newProps = mcf.getConnectionProperties(subject, cri);
	   if (!props.equals(newProps))
	   {
		  throw new ResourceException("Wrong credentials passed to getConnection!");
	   } // end of if ()
	}
		
	/**
	 * @return
	 */
	public Properties getProps() {
		return props;
	}

}
