/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.jca;

import javax.naming.NamingException;
import javax.naming.Reference;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ManagedConnectionFactory;
import org.apache.log4j.Category;
/**
 *
 * @author  Scott.Stark@jboss.org
 * @version $Revision: 1.1.1.1 $
 */
public class WrappedSessionFactoryImpl implements WrappedSessionFactory {
  static Category log = Category.getInstance(WrappedSessionFactoryImpl.class);
  private transient ConnectionManager manager;
  private transient ManagedConnectionFactory factory;
  private transient WrappedSessionRequestInfo ldapInfo;
  private Reference reference;
  WrappedSessionFactoryImpl(
    ConnectionManager manager,
    ManagedConnectionFactory factory,
    WrappedSessionRequestInfo ldapInfo) {
    this.manager = manager;
    this.factory = factory;
    this.ldapInfo = ldapInfo;
    log.debug("ctor, ldapInfo=" + ldapInfo);
  }
  public WrappedSession getMailSession() throws NamingException {
    log.debug("getConnection()");
	 WrappedSession lc = null;
    try {
      lc = (WrappedSession) manager.allocateConnection(factory, ldapInfo);
    } catch (ResourceException e) {
      throw new NamingException("Unable to get Connection: " + e);
    }
    return lc;
  }
  public WrappedSession getMailSession(String user, String password)  
    throws NamingException {
    	
    WrappedSessionRequestInfo newInfo = new WrappedSessionRequestInfo(user, password);	
    	
    log.debug("getConnection, user=" + user + ",password=" + password);
	 WrappedSession lc = null;
    try {
      lc = (WrappedSession) manager.allocateConnection(factory, newInfo);
    } catch (ResourceException e) {
      throw new NamingException("Unable to get Connection: " + e);
    }
    return lc;
  }
  public void setReference(Reference reference) {
    log.debug(
      "setReference, reference=" + reference);
    this.reference = reference;
  }
  public Reference getReference() throws NamingException {
    log.debug("getReference");
    return reference;
  }
}
