/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.jca;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.mail.Session;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;

import org.apache.log4j.Category;

/**
 *
 * @author  Scott.Stark@jboss.org
 * @version $Revision: 1.1.1.1 $
 */
public class WrappedSessionManagedSessionFactory
	implements ManagedConnectionFactory, Serializable {
	static Category log =
		Category.getInstance(WrappedSessionManagedSessionFactory.class);

	private String username;
	private String password;
	
	private boolean _debug = false;

	protected final Properties connectionProps = new Properties();

	/** Creates new LDAPManagedConnectionFactory */
	public WrappedSessionManagedSessionFactory() {
	}

	public Object createConnectionFactory() throws ResourceException {
		if (_debug) {
			log.debug("createConnectionFactory");
		}
		throw new UnsupportedOperationException("Cannot be used in unmanaed env");
	}
	public Object createConnectionFactory(ConnectionManager cm)
		throws ResourceException {
		if (_debug) {
			log.debug("createConnectionFactory, cm=" + cm);
		}

		WrappedSessionRequestInfo lrInfo =
			new WrappedSessionRequestInfo(
				username,
				password);
		return new WrappedSessionFactoryImpl(cm, this, lrInfo);
	}
	//   public ManagedConnection createManagedConnection(Subject subject,
	//      ConnectionRequestInfo info)
	//      throws ResourceException
	//   {
	//      WrappedSessionRequestInfo ldapInfo = (WrappedSessionRequestInfo) info;
	//      return new WrappedManagedSession(subject, ldapInfo);
	//   }
	//   public ManagedConnection matchManagedConnections(Set connectionSet, Subject subject,
	//      ConnectionRequestInfo info)
	//      throws ResourceException
	//   {
	//   	if (_debug) {
	//			log.debug("matchManagedConnections, connectionSet="+connectionSet+", subject="+subject+", info="+info);
	//   	}      
	//      return (ManagedConnection) connectionSet.iterator().next();
	//   }
	//   

	/**
		* Gets full set of connection properties, i.e. whatever is provided
		* in config plus "user" and "password" from subject/cri.
		*
		* <p>Note that the set is used to match connections to datasources as well
		* as to create new managed connections.
		*
		* <p>In fact, we have a problem here. Theoretically, there is a possible
		* name collision between config properties and "user"/"password".
		*/
	protected Properties getConnectionProperties(
		Subject subject,
		ConnectionRequestInfo cri)
		throws ResourceException {
		if (cri != null && cri.getClass() != WrappedSessionRequestInfo.class) {
			throw new ResourceException(
				"Wrong kind of ConnectionRequestInfo: " + cri.getClass());
		} // end of if ()

		// NOTE: we do this because we do not have to synchronize at all on the connectionProps
		// AVOID GLOBAL Synchronizations.  (Bill Burke)
		Properties props = new Properties();
		props.putAll(connectionProps);
		if (subject != null) {
			for (Iterator i = subject.getPrivateCredentials().iterator();
				i.hasNext();
				) {
				Object o = i.next();
				if (o instanceof PasswordCredential
					&& ((PasswordCredential) o)
						.getManagedConnectionFactory()
						.equals(
						this)) {
					PasswordCredential cred = (PasswordCredential) o;
					props.setProperty(
						"user",
						(cred.getUserName() == null) ? "" : cred.getUserName());
					props.setProperty(
						"password",
						new String(cred.getPassword()));
					return props;
				} // end of if ()
			} // end of for ()
			throw new ResourceException("No matching credentials in Subject!");
		} // end of if ()
		WrappedSessionRequestInfo lcri = (WrappedSessionRequestInfo) cri;
		if (lcri != null) {
			props.setProperty(
				"user",
				(lcri.getUserName() == null) ? "" : lcri.getUserName());
			props.setProperty(
				"password",
				(lcri.getPassword() == null) ? "" : lcri.getPassword());
			return props;
		} // end of if ()
		if (username != null) {
			props.setProperty("user", username);
			props.setProperty("password", (password == null) ? "" : password);
		}
		return props;
	}
	
	/**
    *
	 * @param param1 <description>
	 * @param param2 <description>
	 * @return <description>
	 * @exception javax.resource.ResourceException <description>
	 */
	public ManagedConnection createManagedConnection(
		Subject subject,
		ConnectionRequestInfo cri)
		throws javax.resource.ResourceException {
		Properties props = getConnectionProperties(subject, cri);
		//Session session = Session.getDefaultInstance(props);

		// get the initial context and look up java mail
		Session session;
		try {
			InitialContext ic = new InitialContext();
			session = (Session) ic.lookup("java:/Mail");
		} catch (NamingException e) {
			e.printStackTrace();
			throw new ResourceException("can't look up session from JNDI tree");
		}

		return new ManagedWrappedSession(this, session, props);
	}

	/**
	 *
	 * @param param1 <description>
	 * @param param2 <description>
	 * @param param3 <description>
	 * @return <description>
	 * @exception javax.resource.ResourceException <description>
	 */
	public ManagedConnection matchManagedConnections(
		Set mcs,
		Subject subject,
		ConnectionRequestInfo cri)
		throws ResourceException {
		Properties newProps = getConnectionProperties(subject, cri);
		for (Iterator i = mcs.iterator(); i.hasNext();) {
			Object o = i.next();
			if (o instanceof ManagedWrappedSession) {
				ManagedWrappedSession mc = (ManagedWrappedSession) o;
				//if (mc.getProps().equals(newProps) && mc.checkValid())
				if (mc.getProps().equals(newProps)) {
					return mc;
				} // end of if ()

			} // end of if ()
		} // end of for ()
		return null;
	}

	public PrintWriter getLogWriter() throws ResourceException {
		return null;
	}
	public void setLogWriter(PrintWriter out) throws ResourceException {
	}
	public boolean equals(Object other) {
		return super.equals(other);
	}
	public int hashCode() {
		return super.hashCode();
	}

	


	/**
	 * Returns the password.
	 * @return String
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * Returns the username.
	 * @return String
	 */
	public String getUserName() {
		return username;
	}

	/**
	 * Sets the password.
	 * @param password The password to set
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * Sets the username.
	 * @param username The username to set
	 */
	public void setUserName(String username) {
		this.username = username;
	}
}
